@@ -1,14 +1,47 @@
 package com.genymobile.scrcpy;
 
+import com.genymobile.scrcpy.audio.AudioCapture;
+import com.genymobile.scrcpy.audio.AudioCodec;
+import com.genymobile.scrcpy.audio.AudioDirectCapture;
+import com.genymobile.scrcpy.audio.AudioEncoder;
+import com.genymobile.scrcpy.audio.AudioPlaybackCapture;
+import com.genymobile.scrcpy.audio.AudioRawRecorder;
+import com.genymobile.scrcpy.audio.AudioSource;
+import com.genymobile.scrcpy.control.ControlChannel;
+import com.genymobile.scrcpy.control.Controller;
+import com.genymobile.scrcpy.control.DeviceMessage;
+import com.genymobile.scrcpy.device.ConfigurationException;
+import com.genymobile.scrcpy.device.DesktopConnection;
+import com.genymobile.scrcpy.device.Device;
+import com.genymobile.scrcpy.device.Streamer;
+import com.genymobile.scrcpy.util.Ln;
+import com.genymobile.scrcpy.util.LogUtils;
+import com.genymobile.scrcpy.util.Settings;
+import com.genymobile.scrcpy.util.SettingsException;
+import com.genymobile.scrcpy.video.CameraCapture;
+import com.genymobile.scrcpy.video.ScreenCapture;
+import com.genymobile.scrcpy.video.SurfaceCapture;
+import com.genymobile.scrcpy.video.SurfaceEncoder;
+import com.genymobile.scrcpy.video.VideoSource;
+
 import android.os.BatteryManager;
 import android.os.Build;
 
+import java.io.File;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.List;
 
 public final class Server {
 
+    public static final String SERVER_PATH;
+
+    static {
+        String[] classPaths = System.getProperty("java.class.path").split(File.pathSeparator);
+        // By convention, scrcpy is always executed with the absolute path of scrcpy-server.jar as the first item in the classpath
+        SERVER_PATH = classPaths[0];
+    }
+
     private static class Completion {
         private int running;
         private boolean fatalError;
@@ -42,46 +75,47 @@ public final class Server {
         // not instantiable
     }
 
-    private static void initAndCleanUp(Options options) {
-        boolean mustDisableShowTouchesOnCleanUp = false;
-        int restoreStayOn = -1;
-        boolean restoreNormalPowerMode = options.getControl(); // only restore power mode if control is enabled
-        if (options.getShowTouches() || options.getStayAwake()) {
-            if (options.getShowTouches()) {
-                try {
-                    String oldValue = Settings.getAndPutValue(Settings.TABLE_SYSTEM, "show_touches", "1");
-                    // If "show touches" was disabled, it must be disabled back on clean up
-                    mustDisableShowTouchesOnCleanUp = !"1".equals(oldValue);
-                } catch (SettingsException e) {
-                    Ln.e("Could not change \"show_touches\"", e);
-                }
-            }
+    private static void initAndCleanUp(Options options, CleanUp cleanUp) {
+        // This method is called from its own thread, so it may only configure cleanup actions which are NOT dynamic (i.e. they are configured once
+        // and for all, they cannot be changed from another thread)
 
-            if (options.getStayAwake()) {
-                int stayOn = BatteryManager.BATTERY_PLUGGED_AC | BatteryManager.BATTERY_PLUGGED_USB | BatteryManager.BATTERY_PLUGGED_WIRELESS;
-                try {
-                    String oldValue = Settings.getAndPutValue(Settings.TABLE_GLOBAL, "stay_on_while_plugged_in", String.valueOf(stayOn));
-                    try {
-                        restoreStayOn = Integer.parseInt(oldValue);
-                        if (restoreStayOn == stayOn) {
-                            // No need to restore
-                            restoreStayOn = -1;
-                        }
-                    } catch (NumberFormatException e) {
-                        restoreStayOn = 0;
+        if (options.getShowTouches()) {
+            try {
+                String oldValue = Settings.getAndPutValue(Settings.TABLE_SYSTEM, "show_touches", "1");
+                // If "show touches" was disabled, it must be disabled back on clean up
+                if (!"1".equals(oldValue)) {
+                    if (!cleanUp.setDisableShowTouches(true)) {
+                        Ln.e("Could not disable show touch on exit");
                     }
-                } catch (SettingsException e) {
-                    Ln.e("Could not change \"stay_on_while_plugged_in\"", e);
                 }
+            } catch (SettingsException e) {
+                Ln.e("Could not change \"show_touches\"", e);
             }
         }
 
-        if (options.getCleanup()) {
+        if (options.getStayAwake()) {
+            int stayOn = BatteryManager.BATTERY_PLUGGED_AC | BatteryManager.BATTERY_PLUGGED_USB | BatteryManager.BATTERY_PLUGGED_WIRELESS;
             try {
-                CleanUp.configure(options.getDisplayId(), restoreStayOn, mustDisableShowTouchesOnCleanUp, restoreNormalPowerMode,
-                        options.getPowerOffScreenOnClose());
-            } catch (IOException e) {
-                Ln.e("Could not configure cleanup", e);
+                String oldValue = Settings.getAndPutValue(Settings.TABLE_GLOBAL, "stay_on_while_plugged_in", String.valueOf(stayOn));
+                try {
+                    int restoreStayOn = Integer.parseInt(oldValue);
+                    if (restoreStayOn != stayOn) {
+                        // Restore only if the current value is different
+                        if (!cleanUp.setRestoreStayOn(restoreStayOn)) {
+                            Ln.e("Could not restore stay on on exit");
+                        }
+                    }
+                } catch (NumberFormatException e) {
+                    // ignore
+                }
+            } catch (SettingsException e) {
+                Ln.e("Could not change \"stay_on_while_plugged_in\"", e);
+            }
+        }
+
+        if (options.getPowerOffScreenOnClose()) {
+            if (!cleanUp.setPowerOffScreen(true)) {
+                Ln.e("Could not power off screen on exit");
             }
         }
     }
@@ -92,9 +126,13 @@ public final class Server {
             throw new ConfigurationException("Camera mirroring is not supported");
         }
 
-        final Device device = new Device(options);
+        CleanUp cleanUp = null;
+        Thread initThread = null;
 
-        Thread initThread = startInitThread(options);
+        if (options.getCleanup()) {
+            cleanUp = CleanUp.configure(options.getDisplayId());
+            initThread = startInitThread(options, cleanUp);
+        }
 
         int scid = options.getScid();
         boolean tunnelForward = options.isTunnelForward();
@@ -102,9 +140,11 @@ public final class Server {
         boolean video = options.getVideo();
         boolean audio = options.getAudio();
         boolean sendDummyByte = options.getSendDummyByte();
-        boolean camera = options.getVideoSource() == VideoSource.CAMERA;
+        boolean camera = video && options.getVideoSource() == VideoSource.CAMERA;
 
-        Workarounds.apply(audio, camera);
+        final Device device = camera ? null : new Device(options);
+
+        Workarounds.apply();
 
         List<AsyncProcessor> asyncProcessors = new ArrayList<>();
 
@@ -115,14 +155,25 @@ public final class Server {
             }
 
             if (control) {
-                Controller controller = new Controller(device, connection, options.getClipboardAutosync(), options.getPowerOn());
-                device.setClipboardListener(text -> controller.getSender().pushClipboardText(text));
+                ControlChannel controlChannel = connection.getControlChannel();
+                Controller controller = new Controller(device, controlChannel, cleanUp, options.getClipboardAutosync(), options.getPowerOn());
+                device.setClipboardListener(text -> {
+                    DeviceMessage msg = DeviceMessage.createClipboard(text);
+                    controller.getSender().send(msg);
+                });
                 asyncProcessors.add(controller);
             }
 
             if (audio) {
                 AudioCodec audioCodec = options.getAudioCodec();
-                AudioCapture audioCapture = new AudioCapture(options.getAudioSource());
+                AudioSource audioSource = options.getAudioSource();
+                AudioCapture audioCapture;
+                if (audioSource.isDirect()) {
+                    audioCapture = new AudioDirectCapture(audioSource);
+                } else {
+                    audioCapture = new AudioPlaybackCapture(options.getAudioDup());
+                }
+
                 Streamer audioStreamer = new Streamer(connection.getAudioFd(), audioCodec, options.getSendCodecMeta(), options.getSendFrameMeta());
                 AsyncProcessor audioRecorder;
                 if (audioCodec == AudioCodec.RAW) {
@@ -158,7 +209,9 @@ public final class Server {
 
             completion.await();
         } finally {
-            initThread.interrupt();
+            if (initThread != null) {
+                initThread.interrupt();
+            }
             for (AsyncProcessor asyncProcessor : asyncProcessors) {
                 asyncProcessor.stop();
             }
@@ -166,7 +219,9 @@ public final class Server {
             connection.shutdown();
 
             try {
-                initThread.join();
+                if (initThread != null) {
+                    initThread.join();
+                }
                 for (AsyncProcessor asyncProcessor : asyncProcessors) {
                     asyncProcessor.join();
                 }
@@ -178,8 +233,8 @@ public final class Server {
         }
     }
 
-    private static Thread startInitThread(final Options options) {
-        Thread thread = new Thread(() -> initAndCleanUp(options), "init-cleanup");
+    private static Thread startInitThread(final Options options, final CleanUp cleanUp) {
+        Thread thread = new Thread(() -> initAndCleanUp(options, cleanUp), "init-cleanup");
         thread.start();
         return thread;
     }
@@ -224,7 +279,7 @@ public final class Server {
                 Ln.i(LogUtils.buildDisplayListMessage());
             }
             if (options.getListCameras() || options.getListCameraSizes()) {
-                Workarounds.apply(false, true);
+                Workarounds.apply();
                 Ln.i(LogUtils.buildCameraListMessage(options.getListCameraSizes()));
             }
             // Just print the requested data, do not mirror
