@@ -1,8 +1,8 @@
 package com.genymobile.scrcpy.wrappers;
 
 import com.genymobile.scrcpy.FakeContext;
-import com.genymobile.scrcpy.Ln;
-import com.genymobile.scrcpy.SettingsException;
+import com.genymobile.scrcpy.util.Ln;
+import com.genymobile.scrcpy.util.SettingsException;
 
 import android.annotation.SuppressLint;
 import android.content.AttributionSource;
@@ -11,7 +11,6 @@ import android.os.Bundle;
 import android.os.IBinder;
 
 import java.io.Closeable;
-import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 
 public final class ContentProvider implements Closeable {
@@ -42,8 +41,6 @@ public final class ContentProvider implements Closeable {
     private Method callMethod;
     private int callMethodVersion;
 
-    private Object attributionSource;
-
     ContentProvider(ActivityManager manager, Object provider, String name, IBinder token) {
         this.manager = manager;
         this.provider = provider;
@@ -77,8 +74,7 @@ public final class ContentProvider implements Closeable {
         return callMethod;
     }
 
-    private Bundle call(String callMethod, String arg, Bundle extras)
-            throws NoSuchMethodException, InvocationTargetException, IllegalAccessException {
+    private Bundle call(String callMethod, String arg, Bundle extras) throws ReflectiveOperationException {
         try {
             Method method = getCallMethod();
             Object[] args;
@@ -99,7 +95,7 @@ public final class ContentProvider implements Closeable {
                 }
             }
             return (Bundle) method.invoke(provider, args);
-        } catch (InvocationTargetException | IllegalAccessException | NoSuchMethodException e) {
+        } catch (ReflectiveOperationException e) {
             Ln.e("Could not invoke method", e);
             throw e;
         }
